from direct.directbase import DirectStart
from direct.showbase import DirectObject 
from pandac.PandaModules import Vec3
import math

class CameraHandler(DirectObject.DirectObject):
  def __init__(self):
    base.disableMouse()
    self.mx,self.my=0,0
    self.dragging=False
    self.target=Vec3()
    self.camDist=40   
    self.setTarget(0,0,0)
    self.turnCameraAroundPoint(0,0,self.target,self.camDist)
    self.accept("mouse3",self.startDrag)
    self.accept("mouse3-up",self.stopDrag)
    self.accept("wheel_up",lambda : self.adjustCamDist(0.9))
    self.accept("wheel_down",lambda : self.adjustCamDist(1.1))
    taskMgr.add(self.dragTask,'dragTask')    
  def turnCameraAroundPoint(self,tx,ty,p,dist):
        newCamHpr=Vec3()  
        camHpr=base.camera.getHpr()
        newCamHpr.setX(camHpr.getX()+tx)
        newCamHpr.setY(camHpr.getY()-ty)
        newCamHpr.setZ(camHpr.getZ())
        base.camera.setHpr(newCamHpr)
        angleradiansX = newCamHpr.getX() * (math.pi / 180.0)
        angleradiansY = newCamHpr.getY() * (math.pi / 180.0)
        base.camera.setPos(dist*math.sin(angleradiansX)*math.cos(angleradiansY),
                           -dist*math.cos(angleradiansX)*math.cos(angleradiansY),
                           -dist*math.sin(angleradiansY)) 
  def setTarget(self,x,y,z):
    self.target.setX(x)
    self.target.setY(y)
    self.target.setZ(z)
  def startDrag(self):
    self.dragging=True
  def stopDrag(self):
    self.dragging=False
  def adjustCamDist(self,aspect):
    self.camDist=self.camDist*aspect
    self.turnCameraAroundPoint(0,0,self.target,self.camDist)
  def dragTask(self,task):
    if base.mouseWatcherNode.hasMouse():
        mpos = base.mouseWatcherNode.getMouse()  
        if self.dragging:
            self.turnCameraAroundPoint((self.mx-mpos.getX())*100,(self.my-mpos.getY())*100,self.target,self.camDist)        
        self.mx=mpos.getX()
        self.my=mpos.getY()
    return task.cont   